/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtymis.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtymis.custom.service.TfYwpnPipeService;
import com.tofly.xrtymis.entity.TfYwpnPipe;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 管线信息采集数据表：TF_YWPN _PIPE_W
 *
 * @author fcl
 * @date Fri Apr 16 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfywpnpipe")
@Api(tags="管线信息采集数据表：TF_YWPN _PIPE_W接口")
public class TfYwpnPipeController {

    private final TfYwpnPipeService tfYwpnPipeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfYwpnPipe 管线信息采集数据表：TF_YWPN _PIPE_W
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTfYwpnPipePage(Page page, @ModelAttribute TfYwpnPipe tfYwpnPipe) {
      return ResultRespone.success(tfYwpnPipeService.page(page, Wrappers.query(tfYwpnPipe)));
    }

    /**
     * 通过id查询管线信息采集数据表：TF_YWPN _PIPE_W
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(tfYwpnPipeService.getById(id));
    }

    /**
     * 新增管线信息采集数据表：TF_YWPN _PIPE_W
     * @param tfYwpnPipe 管线信息采集数据表：TF_YWPN _PIPE_W
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增管线信息采集数据表：TF_YWPN _PIPE_W" )
    @ApiOperation(value = "新增管线信息采集数据表：TF_YWPN _PIPE_W")
    @PostMapping("/add")
    public ResultRespone save(TfYwpnPipe tfYwpnPipe) {
      return tfYwpnPipeService.savePipeline(tfYwpnPipe);
    }

    /**
     * 修改管线信息采集数据表：TF_YWPN _PIPE_W
     * @param tfYwpnPipe 管线信息采集数据表：TF_YWPN _PIPE_W
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改管线信息采集数据表：TF_YWPN _PIPE_W" )
    @ApiOperation(value = "修改管线信息采集数据表：TF_YWPN _PIPE_W")
    @PostMapping("/update")
    public ResultRespone updateById(List<MultipartFile> fileList, TfYwpnPipe tfYwpnPipe) {
      return tfYwpnPipeService.updatePipeById(fileList,tfYwpnPipe);
    }

    /**
     * 通过id删除管线信息采集数据表：TF_YWPN _PIPE_W
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除管线信息采集数据表：TF_YWPN _PIPE_W" )
    @ApiOperation(value = "通过ID删除管线信息采集数据表：TF_YWPN _PIPE_W")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return tfYwpnPipeService.removePipeById(id);
    }

//    /**
//     * 通过id批量删除管线信息采集数据表：TF_YWPN _PIPE_W
//     * @param ids id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id批量删除管线信息采集数据表：TF_YWPN _PIPE_W" )
//    @ApiOperation(value = "通过ID批量删除管线信息采集数据表：TF_YWPN _PIPE_W")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
//    })
//    @DeleteMapping("/deleteByIds" )
//    public ResultRespone removeById(String ids) {
//        return ResultRespone.success(tfYwpnPipeService.removeByIds(Arrays.asList(ids.split(","))));
//    }
}
