/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtymis.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtymis.custom.service.TfYwpnPipenodeService;
import com.tofly.xrtymis.entity.TfYwpnPipenode;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 管点信息采集数据表：TF_YWPN_PIPENODE_W
 *
 * @author fcl
 * @date Fri Apr 16 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfywpnpipenode")
@Api(tags="管点信息采集数据表：TF_YWPN_PIPENODE_W接口")
public class TfYwpnPipenodeController {

    private final TfYwpnPipenodeService tfYwpnPipenodeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfYwpnPipenode 管点信息采集数据表：TF_YWPN_PIPENODE_W
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTfYwpnPipenodePage(Page page, @ModelAttribute TfYwpnPipenode tfYwpnPipenode) {
      return ResultRespone.success(tfYwpnPipenodeService.page(page, Wrappers.query(tfYwpnPipenode)));
    }

    /**
     * 通过id查询管点信息采集数据表：TF_YWPN_PIPENODE_W
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(tfYwpnPipenodeService.getById(id));
    }

    /**
     * 新增管点信息采集数据表：TF_YWPN_PIPENODE_W
     * @param tfYwpnPipenode 管点信息采集数据表：TF_YWPN_PIPENODE_W
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增管点信息采集数据表：TF_YWPN_PIPENODE_W" )
    @ApiOperation(value = "新增管点信息采集数据表：TF_YWPN_PIPENODE_W")
    @PostMapping("/add")
    public ResultRespone save( TfYwpnPipenode tfYwpnPipenode) {
      return tfYwpnPipenodeService.saveTubePointsInfo(tfYwpnPipenode);
    }

    /**
     * 修改管点信息采集数据表：TF_YWPN_PIPENODE_W
     * @param tfYwpnPipenode 管点信息采集数据表：TF_YWPN_PIPENODE_W
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改管点信息采集数据表：TF_YWPN_PIPENODE_W" )
    @ApiOperation(value = "修改管点信息采集数据表：TF_YWPN_PIPENODE_W")
    @PostMapping("/update")
    public ResultRespone updateById(List<MultipartFile> fileList, TfYwpnPipenode tfYwpnPipenode) {
      return tfYwpnPipenodeService.updateNodeById(fileList,tfYwpnPipenode);
    }

    /**
     * 通过id删除管点信息采集数据表：TF_YWPN_PIPENODE_W
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除管点信息采集数据表：TF_YWPN_PIPENODE_W" )
    @ApiOperation(value = "通过ID删除管点信息采集数据表：TF_YWPN_PIPENODE_W")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return tfYwpnPipenodeService.removeNodeById(id);
    }

//    /**
//     * 通过id批量删除管点信息采集数据表：TF_YWPN_PIPENODE_W
//     * @param ids id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id批量删除管点信息采集数据表：TF_YWPN_PIPENODE_W" )
//    @ApiOperation(value = "通过ID批量删除管点信息采集数据表：TF_YWPN_PIPENODE_W")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
//    })
//    @DeleteMapping("/deleteByIds" )
//    public ResultRespone removeById(String ids) {
//        return ResultRespone.success(tfYwpnPipenodeService.removeByIds(Arrays.asList(ids.split(","))));
//    }

}
