/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtymis.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtymis.custom.service.TfYwpnStationService;
import com.tofly.xrtymis.entity.TfYwpnStation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 监测站信息采集数据表：TF_YWPN_STATION_W
 *
 * @author fcl
 * @date Fri Apr 16 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfywpnstation")
@Api(tags="监测站信息采集数据表：TF_YWPN_STATION_W接口")
public class TfYwpnStationController {

    private final TfYwpnStationService tfYwpnStationService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfYwpnStation 监测站信息采集数据表：TF_YWPN_STATION_W
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTfYwpnStationPage(Page page, @ModelAttribute TfYwpnStation tfYwpnStation) {
      return ResultRespone.success(tfYwpnStationService.page(page, Wrappers.query(tfYwpnStation)));
    }


    /**
     * 通过id查询监测站信息采集数据表：TF_YWPN_STATION_W
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(tfYwpnStationService.getById(id));
    }

    /**
     * 新增监测站信息采集数据表：TF_YWPN_STATION_W
     * @param tfYwpnStation 监测站信息采集数据表：TF_YWPN_STATION_W
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增监测站信息采集数据表：TF_YWPN_STATION_W" )
    @ApiOperation(value = "新增监测站信息采集数据表：TF_YWPN_STATION_W")
    @PostMapping("/add")
    public ResultRespone save( TfYwpnStation tfYwpnStation) {
      return tfYwpnStationService.saveChickInformation(tfYwpnStation);
    }


    /**
     * 修改监测站信息采集数据表：TF_YWPN_STATION_W
     * @param tfYwpnStation 监测站信息采集数据表：TF_YWPN_STATION_W
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改监测站信息采集数据表：TF_YWPN_STATION_W" )
    @ApiOperation(value = "修改监测站信息采集数据表：TF_YWPN_STATION_W")
    @PostMapping("/update")
    public ResultRespone updateById(List<MultipartFile> fileList, TfYwpnStation tfYwpnStation) {
      return tfYwpnStationService.updateStationById(fileList,tfYwpnStation);
    }

    /**
     * 通过id删除监测站信息采集数据表：TF_YWPN_STATION_W
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除监测站信息采集数据表：TF_YWPN_STATION_W" )
    @ApiOperation(value = "通过ID删除监测站信息采集数据表：TF_YWPN_STATION_W")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return tfYwpnStationService.removeStationById(id);
    }

//    /**
//     * 通过id批量删除监测站信息采集数据表：TF_YWPN_STATION_W
//     * @param ids id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id批量删除监测站信息采集数据表：TF_YWPN_STATION_W" )
//    @ApiOperation(value = "通过ID批量删除监测站信息采集数据表：TF_YWPN_STATION_W")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
//    })
//    @DeleteMapping("/deleteByIds" )
//    public ResultRespone removeById(String ids) {
//        return ResultRespone.success(tfYwpnStationService.removeByIds(Arrays.asList(ids.split(","))));
//    }
}
