/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.qualitycontrol.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.qualitycontrol.entity.Massaudit;
import com.tofly.qualitycontrol.service.MassauditService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 质量工单审核记录表
 *
 * @author admin
 * @date Mon Jun 13 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/massaudit")
@Api(tags = "质量工单审核记录表接口")
public class MassauditController {

    private final MassauditService massauditService;

    /**
     * 分页查询
     *
     * @param page      分页对象
     * @param massaudit 质量工单审核记录表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getMassauditPage(Page page, @ModelAttribute Massaudit massaudit) {
        return ResultRespone.success(massauditService.page(page, Wrappers.query(massaudit)));
    }


    /**
     * 通过id查询质量工单审核记录表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(massauditService.getById(id));
    }

    /**
     * 新增质量工单审核记录表
     *
     * @param massaudit 质量工单审核记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增质量工单审核记录表")
    @ApiOperation(value = "新增质量工单审核记录表")
    @PostMapping
    public ResultRespone save(@RequestBody Massaudit massaudit) {
        return ResultRespone.success(massauditService.save(massaudit));
    }

    /**
     * 修改质量工单审核记录表
     *
     * @param massaudit 质量工单审核记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改质量工单审核记录表")
    @ApiOperation(value = "修改质量工单审核记录表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Massaudit massaudit) {
        return ResultRespone.success(massauditService.updateById(massaudit));
    }

    /**
     * 通过id删除质量工单审核记录表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除质量工单审核记录表")
    @ApiOperation(value = "通过ID删除质量工单审核记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(massauditService.removeById(id));
    }

    /**
     * 通过id批量删除质量工单审核记录表
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除质量工单审核记录表")
    @ApiOperation(value = "通过ID批量删除质量工单审核记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(massauditService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
