package com.tofly.pojo.fill;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 月运行记录填报表
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "tf_fill_month_w", resultMap = "fillMonthMap")
@ApiModel
public class FillMonth extends Model<FillMonth> {
    private static final long serialVersionUID = 1L;
    @TableId(value = "ID", type = IdType.ASSIGN_ID)
    @ApiModelProperty(value = "ID，主键")
    private Long id;
    @ApiModelProperty(value = "上报日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date recordTime;
    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    private String scadaTime;
    @ApiModelProperty(value = "上报期号")
    private String issue;
    @ApiModelProperty(value = "联系电话")
    private String phone;
    @ApiModelProperty(value = "本月处理水量(万吨)")
    private String monthDealWq;
    @ApiModelProperty(value = "最高日处理量(万吨/日)")
    private String maxDayDealWq;
    @ApiModelProperty(value = "本月用电量(度)")
    private String monthEc;
    @ApiModelProperty(value = "负荷率(%)")
    private String loadRate;
    @ApiModelProperty(value = "污水厂设计规模(万吨/日)")
    private String spDesignScale;
    @ApiModelProperty(value = "本月直接成本(元/吨)")
    private String monthDirectCost;
    @ApiModelProperty(value = "正常运行天数")
    private String dayNormalOperation;
    @ApiModelProperty(value = "达标排放天数")
    private String reachLetDay;
    @ApiModelProperty(value = "不达标原因")
    private Long notReachSource;
    @ApiModelProperty(value = "不达标原因 转义")
    @TableField(exist = false)
    private String notReachSourceName;
    @ApiModelProperty(value = "超标项目")
    private String exceedItem;
    @ApiModelProperty(value = "是否投加碳源")
    private String cs;
    @ApiModelProperty(value = "是否投加碳源 转义")
    @TableField(exist = false)
    private String csName;
    @ApiModelProperty(value = "投加主要碳源")
    private Long mainCs;
    @ApiModelProperty(value = "投加主要碳源 转义")
    @TableField(exist = false)
    private String mainCsName;
    @ApiModelProperty(value = "投加总量(吨)")
    private String totalCs;
    /**
     * 投加碳源总量(吨)环比
     */
    @ApiModelProperty(value = "投加碳源总量(吨)环比")
    @TableField(exist = false)
    private Float totalCsMoM;
    /**
     * 投加碳源总量(吨)同比
     */
    @ApiModelProperty(value = "投加碳源总量(吨)同比")
    @TableField(exist = false)
    private Float totalCsYoY;
    @ApiModelProperty(value = "有效成分含量(%)")
    private String activeCsRate;
    @ApiModelProperty(value = "是否投加除磷药剂")
    private String pr;
    @ApiModelProperty(value = "是否投加除磷药剂 转义")
    @TableField(exist = false)
    private String prName;
    @ApiModelProperty(value = "投加主要除磷药剂")
    private Long mainPr;
    @ApiModelProperty(value = "投加主要除磷药剂 转义")
    @TableField(exist = false)
    private String mainPrName;
    @ApiModelProperty(value = "投加总量(吨)")
    private String totalPr;
    /**
     * 投加除磷药剂总量(吨)环比
     */
    @ApiModelProperty(value = "投加除磷药剂总量(吨)环比")
    @TableField(exist = false)
    private Float totalPrMoM;
    /**
     * 投加除磷药剂总量(吨)同比
     */
    @ApiModelProperty(value = "投加除磷药剂总量(吨)同比")
    @TableField(exist = false)
    private Float totalPrYoY;
    @ApiModelProperty(value = "有效成分含量(%)")
    private String activePrRate;
    @ApiModelProperty(value = "是否投加脱水药剂")
    private String da;
    @ApiModelProperty(value = "是否投加脱水药剂 转义")
    @TableField(exist = false)
    private String daName;
    @ApiModelProperty(value = "投加主要脱水药剂")
    private Long mainDa;
    @ApiModelProperty(value = "投加主要脱水药剂 转义")
    @TableField(exist = false)
    private String mainDaName;
    @ApiModelProperty(value = "投加总量(吨)")
    private String totalDa;
    /**
     * 投加脱水药剂总量(吨)环比
     */
    @ApiModelProperty(value = "投加脱水药剂总量(吨)环比")
    @TableField(exist = false)
    private Float totalDaMoM;
    /**
     * 投加脱水药剂总量(吨)同比
     */
    @ApiModelProperty(value = "投加脱水药剂总量(吨)同比")
    @TableField(exist = false)
    private Float totalDaYoY;
    @ApiModelProperty(value = "有效成分含量(%)")
    private String activeDaRate;
    @ApiModelProperty(value = "本月脱水污泥生产量(吨/月)")
    private String monthDsPc;
    @ApiModelProperty(value = "污泥含水率(%)")
    private String pcCutRate;
    @ApiModelProperty(value = "本月污泥处置量(吨)")
    private String monthPc;
    @ApiModelProperty(value = "其中,土地利用量(吨)")
    private String landUr;
    @ApiModelProperty(value = "其中,建材利用量(吨)")
    private String bmUr;
    @ApiModelProperty(value = "其中,焚烧利用量(吨)")
    private String ictUr;
    @ApiModelProperty(value = "其中,其他处置量(吨)")
    private String otherUr;
    @ApiModelProperty(value = "不知去向污泥量(吨)")
    private String uwUr;
    @ApiModelProperty(value = "CODcr(mg/L)")
    private String codCr;
    @ApiModelProperty(value = "BOD5(mg/L)")
    private String bod5;
    @ApiModelProperty(value = "NH3-N(mg/L)")
    private String nh3n;
    @ApiModelProperty(value = "SS(mg/L)")
    private String ss;
    @ApiModelProperty(value = "TN(mg/L)")
    private String tn;
    @ApiModelProperty(value = "TP(mg/L)")
    private String tp;
    @ApiModelProperty(value = "PH")
    private String ph;
    @ApiModelProperty(value = "出口CODcr(mg/L)")
    private String codCrOut;
    @ApiModelProperty(value = "出口BOD5(mg/L)")
    private String bod5Out;
    @ApiModelProperty(value = "出口NH3-N(mg/L)")
    private String nh3nOut;
    @ApiModelProperty(value = "出口SS(mg/L)")
    private String ssOut;
    @ApiModelProperty(value = "出口TN(mg/L)")
    private String tnOut;
    @ApiModelProperty(value = "出口TP(mg/L)")
    private String tpOut;
    @ApiModelProperty(value = "出口PH")
    private String phOut;
    @ApiModelProperty(value = "备注")
    private String note;
    @ApiModelProperty(value = "填报单位(部门)")
    private Long fillDept;
    @ApiModelProperty(value = "填报单位(部门)转义")
    @TableField(exist = false)
    private String fillDeptName;
    @ApiModelProperty(value = "所属水厂ID")
    private Long belongStationId;
    @ApiModelProperty(value = "所属水厂ID（多个用,分隔）")
    @TableField(exist = false)
    private String belongStationIds;
    @ApiModelProperty(value = "所属水厂名称转义")
    @TableField(exist = false)
    private String belongStationName;
    @ApiModelProperty(value = "所属组织ID")
    @TableField(exist = false)
    private Long belongDeptId;
    @ApiModelProperty(value = "所属组织名称转义")
    @TableField(exist = false)
    private String belongDeptName;
    @ApiModelProperty(value = "填报日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "FILL_TIME", fill = FieldFill.INSERT)
    private Date fillTime;
    @ApiModelProperty(value = "填报人员")
    @TableField(value = "FILL_USER", fill = FieldFill.INSERT)
    private Long fillUser;
    @ApiModelProperty(value = "填报人员转义")
    @TableField(exist = false)
    private String fillUserName;
    @ApiModelProperty(value = "修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;
    @ApiModelProperty(value = "修改人员")
    @TableField(value = "UPDATE_USER",fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(value = "修改人员转义")
    @TableField(exist = false)
    private String updateUserName;
    @ApiModelProperty(value = "审核人员")
    private Long auditUser;
    @ApiModelProperty(value = "审核人员转义")
    @TableField(exist = false)
    private String auditUserName;
    @ApiModelProperty(value = "审核状态0未审核,1通过,2未通过,3不可审批")
    private String auditState;
    @ApiModelProperty(value = "审核状态转义")
    @TableField(exist = false)
    private String auditStateName;
    @ApiModelProperty(value = "审核时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date auditTime;
    @ApiModelProperty(value = "审核意见说明")
    private String auditNote;
    @ApiModelProperty(value = "是否上报 0否,1是")
    private String report;
    @ApiModelProperty(value = "是否上报 转义")
    @TableField(exist = false)
    private String reportName;
    @ApiModelProperty(value = "上报时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date reportTime;

    @ApiModelProperty(value = "记录状态 0未上报 1待审核 2完成")
    private String state;
    @ApiModelProperty(value = "记录状态转义")
    @TableField(exist = false)
    private String stateName;

    @ApiModelProperty(value = "查询条件开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(exist = false)
    private Date startTime;
    @ApiModelProperty(value = "查询条件结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(exist = false)
    private Date endTime;

    @ApiModelProperty("月份查询 格式YYYY-MM 2023-02")
    @TableField(exist = false)
    private String monthSel;
}
