/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.car;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.service.car.CarFilemmService;
import com.tofly.pojo.car.CarFilemm;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 车辆档案信息管理
 *
 * @author Zhangyu
 * @date Tue Apr 28 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/carfilemm")
@Api(tags="车辆档案信息管理接口")
public class CarFilemmController {

    private final CarFilemmService carFilemmService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param carFilemm 车辆档案信息管理
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getCarFilemmPage(Page page, @ModelAttribute CarFilemm carFilemm) {
      return ResultRespone.success(carFilemmService.page(page, Wrappers.query(carFilemm)));
    }


    /**
     * 通过id查询车辆档案信息管理
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(carFilemmService.getById(id));
    }

    /**
     * 新增车辆档案信息管理
     * @param carFilemm 车辆档案信息管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增车辆档案信息管理" )
    @ApiOperation(value = "新增车辆档案信息管理")
    @PostMapping
    public ResultRespone save(@RequestBody CarFilemm carFilemm) {
      return ResultRespone.success(carFilemmService.save(carFilemm));
    }

    /**
     * 修改车辆档案信息管理
     * @param carFilemm 车辆档案信息管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改车辆档案信息管理" )
    @ApiOperation(value = "修改车辆档案信息管理")
    @PutMapping
    public ResultRespone updateById(@RequestBody CarFilemm carFilemm) {
      return ResultRespone.success(carFilemmService.updateById(carFilemm));
    }

    /**
     * 通过id删除车辆档案信息管理
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除车辆档案信息管理" )
    @ApiOperation(value = "通过ID删除车辆档案信息管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(carFilemmService.removeById(id));
    }

    /**
     * 通过id批量删除车辆档案信息管理
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除车辆档案信息管理" )
    @ApiOperation(value = "通过ID批量删除车辆档案信息管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(carFilemmService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
