/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.car;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.service.car.CarTypeService;
import com.tofly.pojo.car.CarType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 车辆类型管理
 *
 * @author Zhangyu
 * @date Thu Apr 23 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/cartype")
@Api(tags="车辆类型管理接口")
public class CarTypeController {

    private final CarTypeService carTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param carType 车辆类型管理
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getCarTypePage(Page page, @ModelAttribute CarType carType) {
      return ResultRespone.success(carTypeService.page(page, Wrappers.query(carType)));
    }


    /**
     * 通过id查询车辆类型管理
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(carTypeService.getById(id));
    }

    /**
     * 新增车辆类型管理
     * @param carType 车辆类型管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增车辆类型管理" )
    @ApiOperation(value = "新增车辆类型管理")
    @PostMapping
    public ResultRespone save(@RequestBody CarType carType) {
      return ResultRespone.success(carTypeService.save(carType));
    }

    /**
     * 修改车辆类型管理
     * @param carType 车辆类型管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改车辆类型管理" )
    @ApiOperation(value = "修改车辆类型管理")
    @PutMapping
    public ResultRespone updateById(@RequestBody CarType carType) {
      return ResultRespone.success(carTypeService.updateById(carType));
    }

    /**
     * 通过id删除车辆类型管理
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除车辆类型管理" )
    @ApiOperation(value = "通过ID删除车辆类型管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(carTypeService.removeById(id));
    }

    /**
     * 通过id批量删除车辆类型管理
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除车辆类型管理" )
    @ApiOperation(value = "通过ID批量删除车辆类型管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(carTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
