package com.tofly.xrtygis.config;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

@FeignClient("tofly-xrtymeter")
public interface WaterFeign {

    /**
     * 停水区域查询
     * @param page page
     * @param valveId 阀门id
     * @return page
     */
    @GetMapping("/waterGis/viewWaterCut")
    Page viewWaterCut(@SpringQueryMap Page page, @RequestParam("valveId") String valveId);

    /**
     * 客户用水量查询
     * @param khbh 客户编号
     * @return list
     */
    @GetMapping("/waterGis/viewUseWater")
    Page viewUseWater(@RequestParam("khbh")String khbh);

    /**
     * 水表关联查询
     * @param page page
     * @param khbh 客户编号
     * @return page
     */
    @GetMapping("/waterGis/viewWater")
    Page<Map> viewTable(@SpringQueryMap Page page, @RequestParam("khbh") String khbh, @RequestParam("khxm") String khxm, @RequestParam("khdz") String khdz);

    /**
     * 客户用水监控
     * @param page page
     * @return page
     */
    @GetMapping("/waterGis/waterMonitoring")
    Page waterMonitoring(@SpringQueryMap Page page);

    /**
     * 客户历史用水量
     * @param page page
     * @param yhbh 用户编号
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return page
     */
    @GetMapping("/waterGis/useWaterHistory")
    Page useWaterHistory(@SpringQueryMap Page page,@RequestParam("yhbh") String yhbh,@RequestParam("startTime") String startTime ,@RequestParam("endTime") String endTime);

    /**
     * 获取用户最新抄表用水量
     * @param yhbh
     * @return
     */
    @GetMapping("/read/getNewCbsl")
    String getNewCbsl(@RequestParam("yhbh") String yhbh);
}
