package com.tofly.xrtygis.custom.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 调用第三方接口工具类
 *
 * @author FlashWJL
 * @date 2021/5/9 15:51
 */
@Component
public class CallAPIUtil {

    //配置调用远程IP
    @Value("${sxj.address.ipaddress}")
    private String ipaddressInit;

    private static String ipaddress;

    /**
     * 解决被static修饰的变量在使用@Value读取不到配置文件的情况
     * 类加载就加载
     * 被@PostConstruct修饰的方法会在服务器加载Servle的时候运行，
     * 并且只会被服务器执行一次。PostConstruct在构造函数之后执行,init()方法之前执行。
     */
    @PostConstruct
    public void init() {
        ipaddress = ipaddressInit;
    }


    /**
     * 调用第三方的接口方法
     *
     * @param object 第三方接口规定条件实体类
     * @param url    调用的目标接口地址
     * @return map
     */
    public static Map callAPIUtil(Object object, String url) throws Exception {
//        IpAddress ipAddress = new IpAddress();
        Map map = new HashMap();

        Map objectMap = JSONObject.parseObject(JSON.toJSONString(object), Map.class);
        //由于参数加密是指对所有需要传的参数都要加密，所以需要加上这个 SING 参数
        objectMap.put("SING", "00000000");
        //获取签名时的必要参数：Domain
        map.put("Domain", objectMap);
        //转json字符串
        String string = JSON.toJSONString(map);
        JSONObject json = (JSONObject) JSONObject.parse(string);
        //获取签名
        String sing = getSing(json);
        //设置获取后的签名
        objectMap.put("SING", sing);
        //转json字符串
        String info = JSON.toJSONString(objectMap);
        JSONObject jsonData = (JSONObject) JSONObject.parse(info);
        //请求目标接口
        Map data = callAPI(jsonData, ipaddress + url);

        if ("api/Business/GetReadAndFees".equals(url)) {
            List<Map<String, Object>> result = (List<Map<String, Object>>) data.get("data");
            if (result != null && !result.isEmpty()) {
                List<Map<String, Object>> dateList = result.stream()
                        .sorted(Comparator.comparing(m -> m.get("CBYF").toString())).collect(Collectors.toList());
                data.put("data", dateList);
            }
        }

        return data;
    }

    /**
     * 获取签名
     * 不用调用
     *
     * @param date
     * @return
     * @throws Exception
     */
    public static String getSing(JSONObject date) throws Exception {
//        IpAddress ipAddress = new IpAddress();
        HttpClient client = HttpClients.createDefault();
        // 要调用的接口方法
        String url = ipaddress + "api/SignHelper/Sign";
        //建立HttpPost对象
        HttpPost post = new HttpPost(url);
        JSONObject jsonObject = null;
        StringEntity s = new StringEntity(date.toString(), "UTF-8");
        s.setContentEncoding("UTF-8");
        post.addHeader("Content-Type", "application/json; charset=utf-8");
        post.addHeader("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.190 Safari/537.36");
        post.addHeader("Connection", "keep-alive");
        post.addHeader("Accept-Language", "zh-CN,zh;q=0.9");
        post.addHeader("Accept-Encoding", "gzip, deflate");
        post.addHeader("Accept", "application/json");
        post.setEntity(s);
        HttpResponse res = client.execute(post);
        //得到返回的数据
        String response = EntityUtils.toString(res.getEntity());

        if (res.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //将返回数据转换为实体类
            Map singBean = JSONObject.parseObject(response, Map.class);
            //只返回加密的签名
            return singBean.get("message").toString();
        }
        //返回null
        return jsonObject.toString();
    }

    /**
     * 调用目标接口
     * 不用调用
     *
     * @param date
     * @return
     * @throws Exception
     */
    public static Map callAPI(JSONObject date, String inputUrl) throws Exception {
        HttpClient client = HttpClients.createDefault();
        // 要调用的接口路径
        String url = inputUrl;
        //建立HttpPost对象
        HttpPost post = new HttpPost(url);
        Map resultMap = null;
        StringEntity s = new StringEntity(date.toString(), "UTF-8");
        s.setContentEncoding("UTF-8");
        post.addHeader("Content-Type", "application/json");
        post.addHeader("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.190 Safari/537.36");
        post.addHeader("Connection", "keep-alive");
        post.addHeader("Accept-Language", "zh-CN,zh;q=0.9");
        post.addHeader("Accept-Encoding", "gzip, deflate");
        post.addHeader("Accept", "application/json");
        post.setEntity(s);
        HttpResponse res = client.execute(post);
        //得到返回的数据
        String response = EntityUtils.toString(res.getEntity());
        if (res.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //将返回数据转换
            resultMap = JSONObject.parseObject(response, Map.class);
            //返回数据
            return resultMap;
        }
        //返回null
        return resultMap;
    }
}

