/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.build;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;

import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.pojo.build.dto.BuildtypeDto;
import com.tofly.pojo.build.vo.BuildtypeVo;

import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.pojo.build.Buildtype;

import com.tofly.gpsboot.service.build.BuildtypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.validation.Valid;
import java.util.Optional;

/**
 * 工地类型信息表
 *
 * @author admin
 * @date Thu Mar 18 00:00:00 CST 2021
 */
@RestController
@RequestMapping("/buildtype")
@Api(tags = "工地类型信息表接口")
public class BuildtypeController extends BaseController<Buildtype, BuildtypeVo> {

    @Autowired
    private BuildtypeService service;

    @Autowired
    public BuildtypeController(BuildtypeService service) {
        init(service);
    }

    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ToFlyAppLog(title = "分页查询工地类型信息")
    public ResultRespone<Page<BuildtypeVo>> getBuildtypePage(Page page, @ModelAttribute BuildtypeDto parDto) {
        ResultRespone<Page<BuildtypeVo>> rest = new ResultRespone<>();
        try {
            Page<BuildtypeVo> buildtypeVoPage = service.getBuildTypePage(page, parDto);
            rest.setResult(buildtypeVoPage);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }

    @ApiOperation(value = "新增基本信息对象")
    @PostMapping("/save")
    @ToFlyAppLog(title = "新增工地类型信息")
    public ResultRespone<Object> saveBuildtype(@Valid @RequestBody BuildtypeDto parDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        return Optional.ofNullable(super.validRest(rest, result)).orElseGet(() -> super.save(JSON.parseObject(JSON.toJSONString(parDto), Buildtype.class)));

    }

    @ApiOperation(value = "修改基本信息")
    @PutMapping("/updata")
    @ToFlyAppLog(title = "修改工地类型信息")
    public ResultRespone<Object> updateByIdBuildtype(@Valid @RequestBody BuildtypeDto parDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        return Optional.ofNullable(super.validRest(rest, result)).orElseGet(() -> super.updateById(JSON.parseObject(JSON.toJSONString(parDto), Buildtype.class)));

    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @ToFlyAppLog(title = "通过ID查询工地类型信息")
    public ResultRespone<Buildtype> getByIdBuildtype(@PathVariable Long id) {
        return super.getById(id);
    }


    @ApiOperation(value = "通过ID批量删除基本信息对象")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    @ToFlyAppLog(title = "通过ID批量删除工地类型信息")
    public ResultRespone<Object> removeByIdListBuildtype(String ids) {
        ResultRespone<Object> rest = new ResultRespone<>();
        try {
            service.deleteBuildTypeInfo(ids);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }


}
