/*
 * Author: Zev
 * 生成controller
 */
package com.tofly.gpsboot.controller.plan.config;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;

import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.pojo.plan.dto.InspecttypeDto;

import com.tofly.pojo.plan.vo.InspecttypeVo;

import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.pojo.plan.Inspecttype;

import com.tofly.gpsboot.service.config.InspecttypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.validation.Valid;
import java.util.Optional;

/**
 * 巡检类型配置信息
 *
 * @author admin
 * @date Thu Mar 18 00:00:00 CST 2021
 */
@RestController
@RequestMapping("/inspecttype")
@Api(tags = "巡检类型配置信息")
public class InspecttypeController extends BaseController<Inspecttype, InspecttypeVo> {

    @Autowired
    private InspecttypeService service;

    @Autowired
    public InspecttypeController(InspecttypeService service) {
        init(service);
    }

    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ToFlyAppLog(title = "分页查询巡检类型配置信息")
    public ResultRespone<Page<InspecttypeVo>> getInspecttypePage(Page page, @ModelAttribute InspecttypeDto parDto) {
        ResultRespone<Page<InspecttypeVo>> rest = new ResultRespone<>();
        try {
            Page<InspecttypeVo> inspectTypePage = service.getInspectTypePage(page, parDto);
            rest.setResult(inspectTypePage);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;


    }

    @ApiOperation(value = "新增基本信息对象")
    @PostMapping("/save")
    @ToFlyAppLog(title = "新增巡检类型配置信息")
    public ResultRespone<Object> saveInspecttype(@Valid @RequestBody InspecttypeDto parDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                return Optional.ofNullable(super.validRest(rest, result)).orElseGet(() -> super.save(JSON.parseObject(JSON.toJSONString(parDto), Inspecttype.class)));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;

    }

    @ApiOperation(value = "修改基本信息")
    @PutMapping("/updata")
    @ToFlyAppLog(title = "修改巡检类型配置信息")
    public ResultRespone<Object> updateByIdInspecttype(@Valid @RequestBody InspecttypeDto parDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                return Optional.ofNullable(super.validRest(rest, result)).orElseGet(() -> super.updateById(JSON.parseObject(JSON.toJSONString(parDto), Inspecttype.class)));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;

    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @ToFlyAppLog(title = "通过ID查询巡检类型配置信息")
    public ResultRespone<Inspecttype> getByIdInspecttype(@PathVariable Long id) {
        ResultRespone<Inspecttype> rest = new ResultRespone<>();
        try {
            return super.getById(id);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }


    @ApiOperation(value = "通过ID批量删除基本信息对象")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "idList", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    @ToFlyAppLog(title = "通过ID批量删除巡检类型配置信息")
    public ResultRespone<Object> removeByIdListInspecttype(String idList) {
        ResultRespone<Object> rest = new ResultRespone<>();
        try {
            service.deleteInspectTypeInfo(idList);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }


}
