/*
 * Author: Zev
 * 生成controller
 */
package com.tofly.gpsboot.controller.plan.daily;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.pojo.plan.dto.BachPlanCancleDto;
import com.tofly.pojo.plan.dto.PlancancleDto;
import com.tofly.pojo.plan.vo.PlancancleVo;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.paln.task.PlancancleService;
import com.tofly.pojo.plan.Plancancle;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Optional;

/**
 * 巡检计划作废
 *
 * @author admin
 * @date Thu Mar 18 00:00:00 CST 2021
 */
@RestController
@RequestMapping("/planCancle")
@Api(tags = "巡检计划作废")
public class PlanCancleController extends BaseController<Plancancle, PlancancleVo> {

    @Autowired
    private PlancancleService service;

    @Autowired
    public PlanCancleController(PlancancleService planpointService) {
        init(planpointService);
    }

    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ToFlyAppLog(title = "分页查询巡检计划作废信息")
    public ResultRespone<Page<PlancancleVo>> getPlancanclePage(Page page, @ModelAttribute PlancancleDto parDto) {
        ResultRespone<Page<PlancancleVo>> rest = new ResultRespone<>();
        try {
            return super.getInfoPage(page, JSON.parseObject(JSON.toJSONString(parDto), Plancancle.class));
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }

    @ApiOperation(value = "新增基本信息对象")
    @PostMapping("/save")
    @ToFlyAppLog(title = "新增巡检计划作废信息")
    public ResultRespone<Object> savePlancancle(@Valid @RequestBody PlancancleDto parDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                rest.setResult(service.saveInfo(parDto));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;

    }

    @ApiOperation(value = "批量巡检计划作废")
    @PostMapping("/saveBach")
    @ToFlyAppLog(title = "批量巡检计划作废")
    public ResultRespone<Object> saveBachPlanCancle(BachPlanCancleDto dto) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        try {
            List<PlancancleDto> dtoList = dto.getDtoList();
            if(dtoList.size()>0){
                dtoList.forEach(i->service.saveInfo(i));
            }
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }

        return rest;

    }

    @ApiOperation(value = "修改基本信息")
    @PutMapping("/updata")
    @ToFlyAppLog(title = "修改巡检计划作废信息")
    public ResultRespone<Object> updateByIdPlancancle(@Valid @RequestBody PlancancleDto parDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                return super.updateById(JSON.parseObject(JSON.toJSONString(parDto), Plancancle.class));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;

    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @ToFlyAppLog(title = "通过ID查询巡检计划作废信息")
    public ResultRespone<Plancancle> getByIdPlancancle(@PathVariable Long id) {
        ResultRespone<Plancancle> rest = new ResultRespone<>();
        try {
            return super.getById(id);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;

    }


    @ApiOperation(value = "通过ID批量删除基本信息对象")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "idList", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    @ToFlyAppLog(title = "通过ID批量删除巡检计划作废信息")
    public ResultRespone<Object> removeByIdListPlancancle(String idList) {
        ResultRespone<Object> rest = new ResultRespone<>();
        try {
            return super.removeByIdList(idList);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }


}
