/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.trouble;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.controller.BaseController;

import com.tofly.pojo.trouble.dto.ToubleclearDto;
import com.tofly.pojo.trouble.vo.ToubleclearVo;

import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.trouble.ToubleclearService;
import com.tofly.pojo.trouble.Toubleclear;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 隐患消除记录表
 *
 * @author admin
 * @date Thu Mar 18 00:00:00 CST 2021
 */
@RestController
@RequestMapping("/toubleclear")
@Api(tags="隐患消除记录表接口")
public class ToubleclearController extends BaseController<Toubleclear, ToubleclearVo> {
    @Autowired
    private ToubleclearService service;

    @Autowired
    public ToubleclearController(ToubleclearService service) {
        init(service);
    }

    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ToFlyAppLog(title = "分页查询隐患消除记录")
    public ResultRespone<Page<ToubleclearVo>> getToubleclearPage(Page page, @ModelAttribute ToubleclearDto parDto) {
        ResultRespone<Page<ToubleclearVo>> rest = new ResultRespone<>();
        Page<ToubleclearVo> buildPage = service.getToubleclearPage(page, parDto);
        rest.setResult(buildPage);
        return rest;
    }

    @ApiOperation(value = "新增基本信息对象")
    @PostMapping(value = "/save")
    @ToFlyAppLog(title = "新增隐患消除记录")
    public ResultRespone<Object> saveToubleclear(@Valid ToubleclearDto parDto, BindingResult result,@RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            service.savaBuildInfo(fileList, parDto);
        }
        return rest;
    }

    @ApiOperation(value = "修改基本信息")
    @PutMapping("/updata")
    @ToFlyAppLog(title = "修改隐患消除记录")
    public ResultRespone<Object> updateByIdToubleclear(@Valid ToubleclearDto parDto, BindingResult result,@RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                service.updateToubleclearInfo(fileList, parDto);
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }

        }
        return rest;

    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @ToFlyAppLog(title = "通过ID查询隐患消除记录")
    public ResultRespone<Toubleclear> getByIdToubleclear(@PathVariable Long id) {
        return super.getById(id);
    }



    @ApiOperation(value = "通过ID批量删除基本信息对象")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    @ToFlyAppLog(title = "通过ID批量删除隐患消除记录")
    public ResultRespone<Object> removeByIdListToubleclear(String ids) {
        ResultRespone<Object> rest = new ResultRespone<>();
        List<Long> removeId = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        try {

            service.deleteToubleclearInfo(removeId);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }


}
