package com.tofly.spire.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.spire.entity.SpireBigGrid;
import com.tofly.spire.entity.dto.SpireBigGridDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 大表（管网）(SpireBigGrid)表服务接口
 *
 * @author HaiQiu
 * @since 2022-11-11 09:16:06
 */
public interface SpireBigGridService extends IService<SpireBigGrid> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    SpireBigGrid get(Serializable id);

    /**
     * 新增
     *
     * @param spireBigGridDto 实体
     */
    void insert(SpireBigGridDto spireBigGridDto);

    /**
     * 修改
     *
     * @param spireBigGridDto 实体
     */
    void edit(SpireBigGridDto spireBigGridDto);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page         分页
     * @param spireBigGrid 查询wrapper
     * @return 数据集合
     */
    Page<SpireBigGrid> pageList(Page page, SpireBigGrid spireBigGrid);

    /**
     * 数据导出
     *
     * @param page         分页
     * @param spireBigGrid 查询模型
     * @param isAll        是否导出全部数据，true是，false否
     */
    void export(Page page, SpireBigGrid spireBigGrid, Boolean isAll);

    /**
     * 查询所有数据
     *
     * @param spireBigGrid 查询条件
     * @return 查询所有数据
     */
    List<SpireBigGrid> allList(SpireBigGrid spireBigGrid);


    /**
     * 同步管网数据
     * @param date 日期 yyyy-mm-dd
     * @param clean 是否清空原始旧数据
     * @param sn 设备sn
     * @return 同步成功条数
     */
    long getBulkMeterData(String date, Boolean clean, String sn);

    /**
     * 分发管网数据到历史表，指标表等
     * @return
     */
    long synData();
}

