package com.tofly.spire.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.spire.entity.SpireGate;
import com.tofly.spire.entity.dto.SpireGateDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 小表（户表）阀门控制(SpireGate)表服务接口
 *
 * @author HaiQiu
 * @since 2022-11-11 13:40:45
 */
public interface SpireGateService extends IService<SpireGate> {

    /**
     * 小表（户表）阀门控制-通过此接口控制表具阀门的开关(需要表端支持阀门控制)
     * @param sn 表号
     * @param flag true=开阀 false=关阀
     * @return 调用结果
     */
    ResultRespone valveControl(String sn, boolean flag);

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    SpireGate get(Serializable id);

    /**
     * 新增
     *
     * @param spireGateDto 实体
     */
    void insert(SpireGateDto spireGateDto);

    /**
     * 修改
     *
     * @param spireGateDto 实体
     */
    void edit(SpireGateDto spireGateDto);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page      分页
     * @param spireGate 查询wrapper
     * @return 数据集合
     */
    Page<SpireGate> pageList(Page page, SpireGate spireGate);

    /**
     * 数据导出
     *
     * @param page      分页
     * @param spireGate 查询模型
     * @param isAll     是否导出全部数据，true是，false否
     */
    void export(Page page, SpireGate spireGate, Boolean isAll);

    /**
     * 查询所有数据
     *
     * @param spireGate 查询条件
     * @return 查询所有数据
     */
    List<SpireGate> allList(SpireGate spireGate);
}

