package com.tofly.spire.service.impl;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.spire.utils.ResultScada;
import com.tofly.spire.utils.SpireUtils;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.spire.mapper.SpireUserMapper;
import com.tofly.spire.entity.SpireUser;
import com.tofly.spire.entity.dto.SpireUserDto;
import com.tofly.spire.service.SpireUserService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 小表（户表）(SpireUser)表服务实现类
 *
 * @author HaiQiu
 * @since 2022-11-11 13:22:39
 */
@Slf4j
@Service("spireUserService")
public class SpireUserServiceImpl extends ServiceImpl<SpireUserMapper, SpireUser> implements SpireUserService {
    @Autowired
    private SpireUtils spireUtils;


    /**
     * 定时获取-小表（户表）数据接口-通过此接口获取表端上传的数据信息以及日志信息
     * @param params
     * @return
     */
    @XxlJob("getUserInfo")
    public ReturnT getUserInfo(String params){
        ResultScada bulkMeterData = spireUtils.getUserInfo(null);
        List<SpireUser> data = JSONObject.parseArray(bulkMeterData.getData().toString(), SpireUser.class);
        log.info("获取到接口数据：{}条", data.size());
        for (SpireUser datum : data) {
            SpireUser spireUser = this.list(Wrappers.<SpireUser>lambdaQuery()
                    .eq(SpireUser::getSn,datum.getSn())).stream().findFirst().orElse(null);
            if(spireUser!=null && spireUser.getId()>0){
                datum.setId(spireUser.getId());
            }
            this.saveOrUpdate(datum);
        }
        return ReturnT.SUCCESS;
    }

    /**
     * 定时获取-小表（户表）数据接口-通过此接口获取表端上传的数据信息以及日志信息
     * @return
     */
    @Override
    public long getUserInfoData(){
        ResultScada bulkMeterData = spireUtils.getUserInfo(null);
        List<SpireUser> data = JSONObject.parseArray(bulkMeterData.getData().toString(), SpireUser.class);
        log.info("获取到接口数据：{}条", data.size());
        for (SpireUser datum : data) {
            SpireUser spireUser = this.list(Wrappers.<SpireUser>lambdaQuery()
                  .eq(SpireUser::getSn,datum.getSn())).stream().findFirst().orElse(null);
        if(spireUser!=null && spireUser.getId()>0){
            datum.setId(spireUser.getId());
        }
         this.saveOrUpdate(datum);
        }
        return  data.stream().count();
    }


    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public SpireUser get(Serializable id) {
        return getById(id);
    }

    /**
     * 新增
     *
     * @param spireUserDto 实体
     */
    @Override
    public void insert(SpireUserDto spireUserDto) {
        SpireUser spireUser = convert(spireUserDto);
        if (!save(spireUser)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param spireUserDto 实体
     */
    @Override
    public void edit(SpireUserDto spireUserDto) {
        SpireUser spireUser = convert(spireUserDto);
        if (!updateById(spireUser)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param spireUserDto dto实体
     */
    private SpireUser convert(SpireUserDto spireUserDto) {
        SpireUser spireUser = new SpireUser();
        BeanUtils.copyProperties(spireUserDto, spireUser);
        return spireUser;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        SpireUser spireUser = new SpireUser();
        ExcelUtils.createTemplate("小表（户表）", "SpireUser小表（户表）",
                "SpireUser小表（户表）", SpireUser.class, Collections.singletonList(spireUser));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<SpireUser> datas = ExcelUtils.getExcelToList("小表（户表）", file, SpireUser.class);
        if (!CollectionUtils.isEmpty(datas)) {
            saveBatch(datas);
        }
    }

    /**
     * 分页查询
     *
     * @param page      分页
     * @param spireUser 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<SpireUser> pageList(Page page, SpireUser spireUser) {
        return page(page, new QueryWrapper<>(spireUser)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page      分页
     * @param spireUser 查询模型
     * @param isAll     是否导出全部数据，true是，false否
     */
    @Override
    public void export(Page page, SpireUser spireUser, Boolean isAll) {
        Page<SpireUser> pageData = new Page<>();
        if (isAll == null || !isAll) {
            pageData = page(page, new QueryWrapper<>(spireUser)).addOrder(page.getOrders());
        } else {
            pageData.setRecords(list(new QueryWrapper<>(spireUser)));
        }
        //导出数据
        ExcelUtils.getListToExcel(SpireUser.class, pageData.getRecords(), "小表（户表）");
    }

    /**
     * 查询所有数据
     *
     * @param spireUser 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<SpireUser> allList(SpireUser spireUser) {
        return list(new QueryWrapper<>(spireUser));
    }
}

