package com.tofly.water.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.water.entity.DeviceBusiness;
import com.tofly.water.entity.dto.DeviceBusinessDto;
import com.tofly.water.service.DeviceBusinessService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import com.tofly.common.core.entity.ResultRespone;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import com.tofly.common.log.annotation.ToFlyAppLog;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 设备档案和外部主键关联(DeviceBusiness)表控制层
 *
 * @author HaiQiu
 * @since 2022-11-30 15:36:46
 */
@Api(tags = "设备档案和外部主键关联",hidden = true)
@RestController
@RequestMapping("deviceBusiness")
public class DeviceBusinessController {
    /**
     * 服务对象
     */
    @Resource
    private DeviceBusinessService deviceBusinessService;

    /**
     * 分页查询数据
     *
     * @param page           分页对象
     * @param deviceBusiness 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-分页查询数据")
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone<Page<DeviceBusiness>> pageList(Page page, DeviceBusiness deviceBusiness) {
        return ResultRespone.success(deviceBusinessService.pageList(page, deviceBusiness), "查询成功");
    }

    /**
     * 查询所有数据
     *
     * @param deviceBusiness 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-查询所有数据")
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone<List<DeviceBusiness>> allList(DeviceBusiness deviceBusiness) {
        return ResultRespone.success(deviceBusinessService.allList(deviceBusiness), "查询成功");
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-通过主键查询单条数据")
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone<DeviceBusiness> get(@PathVariable Serializable id) {
        return ResultRespone.success(deviceBusinessService.get(id), "查询成功");
    }

    /**
     * 新增数据
     *
     * @param deviceBusinessDto 实体
     * @return 新增结果
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-新增数据")
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@Validated @RequestBody DeviceBusinessDto deviceBusinessDto) {
        deviceBusinessService.insert(deviceBusinessDto);
        return ResultRespone.success(null, "新增成功");
    }

    /**
     * 修改数据
     *
     * @param deviceBusinessDto 实体
     * @return 修改结果
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-修改数据")
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@Validated @RequestBody DeviceBusinessDto deviceBusinessDto) {
        deviceBusinessService.edit(deviceBusinessDto);
        return ResultRespone.success(null, "修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-删除数据")
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        deviceBusinessService.delete(idList);
        return ResultRespone.success(null, "删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-模版下载")
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        deviceBusinessService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-Excel数据导入")
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        deviceBusinessService.importData(file);
        return ResultRespone.success(null, "导入成功");
    }


    /**
     * 数据导出
     *
     * @param page           分页
     * @param deviceBusiness 查询模型
     * @param isAll          是否导出全部数据，true是，false否
     */
    @ToFlyAppLog(title = "设备档案和外部主键关联-数据导出")
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false否（默认否）"),
    })
    @GetMapping("/export")
    public void export(Page page, DeviceBusiness deviceBusiness, boolean isAll) {
        deviceBusinessService.export(page, deviceBusiness, isAll);
    }
}

