package com.tofly.water.entity;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import com.tofly.common.constant.SqlCondition;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;
import com.haiqiu.tools.excel.annotation.Excel;

/**
 * 泵站信息管理(BasePump)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-22 13:57:02
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@TableName(value = "TF_BASE_PUMP", resultMap = "BasePumpMap")
@ApiModel(value = "BasePump对象", description = "泵站信息管理")
public class BasePump extends Model<BasePump> {

    private static final long serialVersionUID = -42692002523694923L;

    /**
     * 主键ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "主键ID")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;
    /**
     * 泵站名称
     */
    @Excel(name = "泵站名称")
    @ApiModelProperty(value = "泵站名称")
    @TableField(condition = SqlCondition.ORACLE_LIKE)
    private String name;
    /**
     * 泵站类型
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "泵站类型")
    @ApiModelProperty(value = "泵站类型")
    private Long category;
    /**
     * 分组
     */
    @Excel(name = "分组")
    @ApiModelProperty(value = "分组")
    private String grouping;
    /**
     * 编号
     */
    @Excel(name = "编号")
    @ApiModelProperty(value = "编号")
    private String code;
    /**
     * 供区名称
     */
    @Excel(name = "供区名称")
    @ApiModelProperty(value = "供区名称")
    @TableField(condition = SqlCondition.ORACLE_LIKE)
    private String support;
    /**
     * 供水规模
     */
    @Excel(name = "供水规模")
    @ApiModelProperty(value = "供水规模")
    private String scale;
    /**
     * 所属区域
     */
    @Excel(name = "所属区域")
    @ApiModelProperty(value = "所属区域")
    private String ascription;
    /**
     * 安装位置
     */
    @Excel(name = "安装位置")
    @ApiModelProperty(value = "安装位置")
    @TableField(condition = SqlCondition.ORACLE_LIKE)
    private String installationPosition;
    /**
     * 坐标(经度，纬度)
     */
    @Excel(name = "坐标(经度，纬度)")
    @ApiModelProperty(value = "坐标(经度，纬度)")
    private String coordinate;

    /**
     * 范围
     */
    @ApiModelProperty(value = "范围")
    private String relativeRange;

    /**
     * 详细地址
     */
    @Excel(name = "详细地址")
    @ApiModelProperty(value = "详细地址")
    private String address;
    /**
     * 备注
     */
    @Excel(name = "备注")
    @ApiModelProperty(value = "备注")
    private String remark;
    /**
     * 投用日期
     */
    @Excel(name = "投用日期")
    @ApiModelProperty(value = "投用日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date investmentDate;
    /**
     * 管理单位
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "管理单位")
    @ApiModelProperty(value = "管理单位")
    private Long managementUnit;
    /**
     * 系统型式
     */
    @Excel(name = "系统型式")
    @ApiModelProperty(value = "系统型式")
    private String model;
    /**
     * 供水能力
     */
    @Excel(name = "供水能力")
    @ApiModelProperty(value = "供水能力")
    private String waterCapacity;
    /**
     * 供应商简称
     */
    @Excel(name = "供应商简称")
    @ApiModelProperty(value = "供应商简称")
    private String supplierSimple;
    /**
     * 供应商全称
     */
    @Excel(name = "供应商全称")
    @ApiModelProperty(value = "供应商全称")
    private String supplierDetail;
    /**
     * 供应商联系方式
     */
    @Excel(name = "供应商联系方式")
    @ApiModelProperty(value = "供应商联系方式")
    private String supplierPhone;
    /**
     * 附件
     */
    @Excel(name = "附件")
    @ApiModelProperty(value = "附件")
    private String annex;
    /**
     * 排序
     */
    @Excel(name = "排序")
    @ApiModelProperty(value = "排序")
    private Long sort;
    /**
     * 删除，0未删除，1已删除
     */
    @Excel(name = "删除，0未删除，1已删除")
    @TableLogic
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;
    /**
     * 创建人
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "创建人")
    @ApiModelProperty(value = "创建人")
    private Long createdUser;
    @TableField(exist = false)
    @ApiModelProperty(value = "创建人", hidden = true)
    private String createdUserName;
    /**
     * 创建时间
     */
    @Excel(name = "创建时间")
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createdTime;
    /**
     * 更新人
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "更新人")
    @ApiModelProperty(value = "更新人")
    private Long updatedUser;
    @TableField(exist = false)
    @ApiModelProperty(value = "更新人", hidden = true)
    private String updatedUserName;
    /**
     * 更新时间
     */
    @Excel(name = "更新时间")
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updatedTime;

    @TableField(exist = false)
    @ApiModelProperty(value = "泵组数据集合", hidden = true)
    private List<BasePumpGroup> basePumpGroups;
}

