package com.tofly.water.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 泵站信息管理(BasePump)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-22 13:57:02
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "BasePump对象", description = "泵站信息管理")
public class BasePumpDto implements Serializable {

    private static final long serialVersionUID = -47585970093836507L;

    /**
     * 主键ID
     */
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 泵站名称
     */
    @Size(max = 85, message = "泵站名称长度不能超过85")
    @NotBlank(message = "泵站名称不能为空")
    @ApiModelProperty(value = "泵站名称")
    private String name;

    /**
     * 泵站类型
     */
    @Digits(integer = 20, fraction = 0, message = "泵站类型长度不能超过20")
    @NotNull(message = "泵站类型不能为空")
    @ApiModelProperty(value = "泵站类型")
    private Long category;

    /**
     * 分组
     */
    @Size(max = 85, message = "分组长度不能超过85")
    @ApiModelProperty(value = "分组")
    private String grouping;

    /**
     * 编号
     */
    @Size(max = 85, message = "编号长度不能超过85")
    @ApiModelProperty(value = "编号")
    private String code;

    /**
     * 供区名称
     */
    @Size(max = 85, message = "供区名称长度不能超过85")
    @ApiModelProperty(value = "供区名称")
    private String support;

    /**
     * 供水规模
     */
    @Size(max = 85, message = "供水规模长度不能超过85")
    @ApiModelProperty(value = "供水规模")
    private String scale;

    /**
     * 所属区域
     */
    @Size(max = 85, message = "所属区域长度不能超过85")
    @ApiModelProperty(value = "所属区域")
    private String ascription;

    /**
     * 安装位置
     */
    @Size(max = 85, message = "安装位置长度不能超过85")
    @ApiModelProperty(value = "安装位置")
    private String installationPosition;

    /**
     * 坐标(经度，纬度)
     */
    @Size(max = 85, message = "坐标(经度，纬度)长度不能超过85")
    @ApiModelProperty(value = "坐标(经度，纬度)")
    private String coordinate;

    /**
     * 范围
     */
    @Size(max = 85, message = "范围长度不能超过85")
    @ApiModelProperty(value = "范围")
    private String relativeRange;

    /**
     * 详细地址
     */
    @Size(max = 85, message = "详细地址长度不能超过85")
    @ApiModelProperty(value = "详细地址")
    private String address;

    /**
     * 备注
     */
    @Size(max = 85, message = "备注长度不能超过85")
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 投用日期
     */
    @ApiModelProperty(value = "投用日期")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date investmentDate;

    /**
     * 管理单位
     */
    @Digits(integer = 20, fraction = 0, message = "管理单位长度不能超过20")
    @ApiModelProperty(value = "管理单位")
    private Long managementUnit;

    /**
     * 系统型式
     */
    @Size(max = 85, message = "系统型式长度不能超过85")
    @ApiModelProperty(value = "系统型式")
    private String model;

    /**
     * 供水能力
     */
    @Size(max = 85, message = "供水能力长度不能超过85")
    @ApiModelProperty(value = "供水能力")
    private String waterCapacity;

    /**
     * 供应商简称
     */
    @Size(max = 85, message = "供应商简称长度不能超过85")
    @ApiModelProperty(value = "供应商简称")
    private String supplierSimple;

    /**
     * 供应商全称
     */
    @Size(max = 300, message = "供应商全称长度不能超过300")
    @ApiModelProperty(value = "供应商全称")
    private String supplierDetail;

    /**
     * 供应商联系方式
     */
    @Size(max = 85, message = "供应商联系方式长度不能超过85")
    @ApiModelProperty(value = "供应商联系方式")
    private String supplierPhone;

    /**
     * 附件
     */
   @ApiModelProperty(value = "附件")
    private String annex;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 删除，0未删除，1已删除
     */
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;

    /**
     * 创建人
     */
    @Digits(integer = 20, fraction = 0, message = "创建人长度不能超过20")
    @ApiModelProperty(value = "创建人")
    private Long createdUser;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createdTime;

    /**
     * 更新人
     */
    @Digits(integer = 20, fraction = 0, message = "更新人长度不能超过20")
    @ApiModelProperty(value = "更新人")
    private Long updatedUser;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updatedTime;

}

