package com.tofly.water.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.water.entity.BasePump;
import com.tofly.water.entity.dto.BasePumpDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 泵站信息管理(BasePump)表服务接口
 *
 * @author HaiQiu
 * @since 2022-11-22 11:34:20
 */
public interface BasePumpService extends IService<BasePump> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    BasePump get(Serializable id);

    /**
     * 新增
     *
     * @param basePumpDto 实体
     * @param files       上传文件附件
     */
    void insert(BasePumpDto basePumpDto, List<MultipartFile> files);

    /**
     * 修改
     *
     * @param basePumpDto 实体
     * @param files       上传文件附件
     */
    void edit(BasePumpDto basePumpDto, List<MultipartFile> files);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page     分页
     * @param basePump 查询wrapper
     * @return 数据集合
     */
    Page<BasePump> pageList(Page page, BasePump basePump);

    /**
     * 数据导出
     *
     * @param page     分页
     * @param basePump 查询模型
     * @param isAll    是否导出全部数据，true是，false否
     */
    void export(Page page, BasePump basePump, Boolean isAll);

    /**
     * 查询所有数据
     *
     * @param basePump 查询条件
     * @return 查询所有数据
     */
    List<BasePump> allList(BasePump basePump);
}

